%==========================================================================
%
%   prioritySampling.m
%
%   Authors: SUMO Lab Team
%   Version: 1.0
%   Copyright: 2017
%   
%   When referring to ALBATROS please make reference to the corresponding publication:
%   
%   Van Steenkiste T., J. van der Herten, D. Deschrijver and T. Dhaene.
%   "ABLATROS: Adaptive Line-Based Sampling Trajectories for Sequential
%   Measurements". Engineering with Computers.
%
%==========================================================================
%
%   SIGNATURE:
%   wayPointsToTravel = prioritySampling( previousSamples,pathCorners, stepSize, distanceFunction )
%
%==========================================================================
%
%   PURPOSE:
%   Perform the priority sampling procedure on a path segment
%
%   INPUT:
%   previousSamples: previously selected samples in the ALBATROS algorithm
%   pathCorners: corners of the path on which to perform priority sampling
%   stepSize: suggested sampling distance
%   distanceFunction: function which computes a distance between two points
%
%   OUTPUT:
%   wayPointsToTravel: selected points to take measurements at
%   %==========================================================================

function wayPointsToTravel = prioritySampling( previousSamples,pathCorners, stepSize, distanceFunction )
	pathDistances = zeros(size(pathCorners,1)-1,1);
	for i = 1:(size(pathCorners,1)-1)
		V1 = pathCorners(i,:);
		V2 = pathCorners(i+1,:);
		pathDistances(i) = distanceFunction(V1,V2);
	end
	totalPathLength = sum(pathDistances);
	numPointsToTake = round(totalPathLength / stepSize);
	wayPointsToTravel = [];
	sizeSample = 1000;
	samples = 0:1/sizeSample:1;
	positions = zeros(numPointsToTake,1);
	for j = 1:numPointsToTake
		maximalDistance = 0;
		P = [];
		for i = 1:length(samples)
			samplePoint = getPointOnPath(samples(i),pathCorners,pathDistances);
			[ ~, closestDistance ] = closestPointToPoint( samplePoint, [previousSamples; wayPointsToTravel], distanceFunction );
			if closestDistance > maximalDistance
				maximalDistance = closestDistance;
				P = samplePoint;
				positions(j) = i;
			end
		end
		wayPointsToTravel = [wayPointsToTravel; P];
	end
	[~,idx] = sort(positions);
	wayPointsToTravel = wayPointsToTravel(idx,:);
	%what if nodesToTravel empty: handle!
	if isempty(wayPointsToTravel)
		wayPointsToTravel = pathCorners(end,:);
	else
		if wayPointsToTravel(end,:) ~= pathCorners(end,:)
			wayPointsToTravel = [wayPointsToTravel; pathCorners(end,:)];
		end
	end
end

